/* Copyright (c) 2016, Jan Wiśniewski <vuko@hackerspace.pl>
 * 
 * This software is provided 'as-is', without any express or implied
 * warranty. In no event will the authors be held liable for any damages
 * arising from the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 */

#include <libopencm3/stm32/dbgmcu.h>
#include <libopencm3/stm32/gpio.h>
#include <libopencm3/stm32/rcc.h>
#include <libopencm3/stm32/rtc.h>
#include <libopencm3/stm32/usart.h>
#include <libopencm3/cm3/nvic.h>
#include <libopencmsis/core_cm3.h>
#include <stdint.h>
#include <stdio.h>

#include "log.h"

void stdio_init(void);
int main(void);

void mdb_slave_init() {
    nvic_enable_irq(NVIC_USART2_IRQ);

	rcc_periph_clock_enable(RCC_GPIOA);
    rcc_periph_clock_enable(RCC_USART2);
	gpio_set_mode(GPIOA, GPIO_MODE_OUTPUT_50_MHZ,
		      GPIO_CNF_OUTPUT_ALTFN_PUSHPULL, GPIO_USART2_TX);
    gpio_set_mode(GPIOA, GPIO_MODE_INPUT,
              GPIO_CNF_INPUT_PULL_UPDOWN, GPIO_USART2_RX);
    gpio_set(GPIOA, GPIO3);
    usart_set_baudrate(USART2, 9600);
    usart_set_databits(USART2, 9);
    usart_set_stopbits(USART2, USART_STOPBITS_1);
    usart_set_parity(USART2, USART_PARITY_NONE);
    usart_set_flow_control(USART2, USART_FLOWCONTROL_NONE);
    usart_set_mode(USART2, USART_MODE_TX_RX);
    usart_enable_rx_interrupt(USART2);
    usart_enable(USART2);
}

static void led_init(void) {
	gpio_set(GPIOC, GPIO13);

	/* Setup GPIO6 and 7 (in GPIO port A) for LED use. */
	gpio_set_mode(GPIOC, GPIO_MODE_OUTPUT_50_MHZ,
		      GPIO_CNF_OUTPUT_PUSHPULL, GPIO13);
}

int main(){
    // enable dubbuger during sleep modes
    //DBGMCU_CR |= DBGMCU_CR_SLEEP | DBGMCU_CR_STOP | DBGMCU_CR_STANDBY;
    stdio_init();

    /* clocks init */
	rcc_clock_setup_in_hse_8mhz_out_72mhz();
	rcc_periph_clock_enable(RCC_GPIOB);
    rcc_periph_clock_enable(RCC_GPIOC);
	rcc_periph_clock_enable(RCC_AFIO);
	rcc_periph_clock_enable(RCC_TIM2);

    led_init();
    mdb_slave_init();

    rtc_auto_awake(RCC_LSE, 0x7fff / 256); // 0x7fff);

    gpio_toggle(GPIOC, GPIO13);
    while(1){
        //log("Testing shit around 1");
        /*usart_send_blocking(USART2, 0b100000000);
        usart_send_blocking(USART2, 1);
        usart_send_blocking(USART2, 2);
        usart_send_blocking(USART2, 3);
        usart_send_blocking(USART2, 4);*/
        uint8_t buf[2];
        uint8_t sz = 0;
        if ((sz = _read(0, buf, 2)) == 2) {
            // log("got bytes %d", sz);
            usart_send_blocking(USART2, buf[0] | (buf[1] << 8));
        }
        // log("tick");
        // usart_send_blocking(USART2, 5);
		// for (int i = 0; i < 8000000; i++)	/* Wait a bit. */
		// 	__asm__("nop");
    }
    return 0;
}

uint8_t mdb_pos = 0;
uint8_t mdb_buffer[40];

void usart2_isr(void)
{
	static uint16_t data = 'A';

	/* Check if we were called because of RXNE. */
	if (((USART_CR1(USART2) & USART_CR1_RXNEIE) != 0) &&
	    ((USART_SR(USART2) & USART_SR_RXNE) != 0)) {

		/* Indicate that we got data. */
		gpio_toggle(GPIOC, GPIO13);

		/* Retrieve the data from the peripheral. */
		data = usart_recv(USART2);

        if (data & (1 << 8)) {
            mdb_pos = 0;
        }

        if (mdb_pos >= 40) {
            log("ovf");
            return;
        }

        mdb_buffer[mdb_pos++] = data & 0xff;

        uint8_t buf[2] = {data & 0xff, data >> 8};
        fwrite(buf, 1, 2, stdout);
        fflush(stdout);
        // log("got byte: %03x [%d]", data, mdb_pos);
	}
}
